'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _schema_route = require('./schema_route');

var _schema_route2 = _interopRequireDefault(_schema_route);

var _nodeBits = require('node-bits');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// helpers
var defineHandler = function defineHandler(key, database, schema, subscribers) {
  return new _schema_route2.default(key, database, schema, subscribers);
};
var defineRoute = function defineRoute(verb, route, implementation) {
  return { verb: verb, route: route, implementation: implementation };
};
var defineSubscribers = function defineSubscribers(subscribers, key) {
  return _lodash2.default.filter(subscribers, function (s) {
    return s.subscribe && s.subscribe(key);
  });
};

var defineRoutes = function defineRoutes(prefix, key, database, schema, subscribers) {
  var applicableSubscribers = defineSubscribers(subscribers, key);
  var handler = defineHandler(key, database, schema, applicableSubscribers);
  var route = (prefix ? '/' + prefix : '') + '/' + key;

  return [defineRoute(_nodeBits.GET, route, handler), defineRoute(_nodeBits.GET, route + '/:id', handler), defineRoute(_nodeBits.POST, route, handler), defineRoute(_nodeBits.POST, route + '/:id', handler), defineRoute(_nodeBits.PUT, route, handler), defineRoute(_nodeBits.PUT, route + '/:id', handler), defineRoute(_nodeBits.DELETE, route, handler), defineRoute(_nodeBits.DELETE, route + '/:id', handler)];
};

// load route

exports.default = function (config) {
  var subscribers = (config.subscribers || []).map(function (s) {
    return s.implementation;
  });
  var routes = _lodash2.default.keys(config.database.models).map(function (key) {
    return defineRoutes(config.prefix, key, config.database, config.schema, subscribers);
  });

  return _lodash2.default.flattenDeep(routes);
};