'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _classAutobind = require('class-autobind');

var _classAutobind2 = _interopRequireDefault(_classAutobind);

var _nodeBits = require('node-bits');

var _schema = require('./schema');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var isPromise = function isPromise(obj) {
  return !!obj && ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' || typeof obj === 'function') && typeof obj.then === 'function';
}; // eslint-disable-line

var SchemaRoute = function () {
  function SchemaRoute(name, database, schema, subscribers) {
    _classCallCheck(this, SchemaRoute);

    (0, _classAutobind2.default)(this);

    this.name = name;
    this.database = database;
    this.schema = schema;
    this.subscribers = subscribers;

    this.logic = {
      get: (0, _schema.get)(name, database),
      post: (0, _schema.post)(name, database),
      put: (0, _schema.put)(name, database),
      delete: (0, _schema.restDelete)(name, database)
    };
  }

  _createClass(SchemaRoute, [{
    key: 'buildParams',
    value: function buildParams(verb, req, res) {
      var _this = this;

      return {
        database: this.database,
        name: this.name,
        schema: this.schema,

        execute: function execute() {
          return _this.logic[verb.toLowerCase()](req, res);
        },

        verb: verb, req: req, res: res,

        stage: _nodeBits.BEFORE,

        resultSent: false,
        failure: false,
        data: null
      };
    }
  }, {
    key: 'notifySubscribers',
    value: function notifySubscribers(params) {
      var series = this.subscribers.map(function (sub) {
        return function () {
          var result = sub.perform(params);

          if (!result) {
            return Promise.resolve();
          }

          if (isPromise(result)) {
            return result.then(function (resultSent) {
              if (resultSent === true) {
                params.resultSent = true;
              }
            });
          }

          if (result === true) {
            params.resultSent = true;
          }

          return Promise.resolve();
        };
      });

      return (0, _nodeBits.executeSeries)(series || []).then(function () {
        return params;
      });
    }
  }, {
    key: 'execute',
    value: function execute(params) {
      if (params.resultSent) {
        return Promise.resolve(params);
      }

      return params.execute().then(function (data) {
        params.data = data;
        params.stage = _nodeBits.AFTER;

        return params;
      });
    }
  }, {
    key: 'send',
    value: function send(params) {
      // send result to client
      if (!params.resultSent) {
        params.res.json(params.data);
        params.resultSent = true;
      }

      return params;
    }
  }, {
    key: 'respond',
    value: function respond(verb, req, res) {
      var params = this.buildParams(verb, req, res);

      this.notifySubscribers(params).then(this.execute).then(this.notifySubscribers).then(this.send).catch(function (err) {
        (0, _nodeBits.logError)(err);

        if (!params.resultSent) {
          res.status(500).send(err);
        }
      });
    }

    // REST

  }, {
    key: 'get',
    value: function get(req, res) {
      this.respond(_nodeBits.GET, req, res);
    }
  }, {
    key: 'post',
    value: function post(req, res) {
      // allow this to happen (you can't put files so we have to support it)
      if (req.params.id || req.body.id) {
        this.respond(_nodeBits.PUT, req, res);
        return;
      }

      this.respond(_nodeBits.POST, req, res);
    }
  }, {
    key: 'put',
    value: function put(req, res) {
      this.respond(_nodeBits.PUT, req, res);
    }
  }, {
    key: 'delete',
    value: function _delete(req, res) {
      this.respond(_nodeBits.DELETE, req, res);
    }
  }]);

  return SchemaRoute;
}();

exports.default = SchemaRoute;